@@ -25,8 +25,11 @@
 #include <libglom/data_structure/glomconversions.h> //For util_build_sql
 #include <libglom/utils.h>
 #include <libglom/db_utils.h>
+#include <libgdamm/datamodel.h>
+#include <libgdamm/dataaccesswrapper.h>
+#include <libgdamm/datamodelarray.h>
 
-#include "glom/application.h"
+#include <glom/application.h>
 
 namespace Glom
 {
@@ -67,9 +70,10 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
         const int cols_count = model.m_data_model_columns_count;
         for(int i = 0; i < cols_count; ++i)
         {
+          //TODO: Use iter->get_value_at(i) with try/catch instead when we can depend on libgdamm >= 4.99.4.2
           Glib::RefPtr<const Gnome::Gda::Holder> holder = iter->get_holder_for_field(i);
           if(holder)
-            m_db_values[i] = holder->get_value(); //TODO_gda: Why not just use get_value_at()?
+            m_db_values[i] = holder->get_value();
           else
           {
             // This is quite possible, for example for unset dates. jhs
@@ -79,6 +83,7 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
           //std::cout << "  debug: col=" << i << ", GType=" << m_db_values[i].get_value_type() << ", string=" << m_db_values[i].to_string() << std::endl;
         }
 
+        //TODO: Use iter->get_value_at(i) with try/catch instead when we can depend on libgdamm >= 4.99.4.2
         Glib::RefPtr<const Gnome::Gda::Holder> holder = iter->get_holder_for_field(model.m_column_index_key);
         if(holder)
           m_key = holder->get_value();  //TODO_gda: Why not just use get_value_at()?
@@ -106,16 +111,24 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
 
       //Create default values, if necessary, of the correct types:
       //Examine the columns in the returned DataModel:
+      const Glib::RefPtr<const Gnome::Gda::DataModel> datamodel = model.m_gda_datamodel;
       for(guint col = 0; col < model.m_data_model_columns_count; ++col)
       {
         if(m_db_values.find(col) == m_db_values.end()) //If there is not already a value in the map for this column.
         {
-          Glib::RefPtr<Gnome::Gda::Column> column = model.m_gda_datamodel->describe_column(col);
+          if(!datamodel) //though this should not happen.
+          {
+            m_db_values[col] = Gnome::Gda::Value();
+          }
+          else
+          {
+            const Glib::RefPtr<const Gnome::Gda::Column> column = datamodel->describe_column(col);
 
-          //We don't just create a Gda::Value of the column's gda type,
-          //because we should use a NULL-type Gda::Value as the initial value for some fields:
-          const Field::glom_field_type glom_type = Field::get_glom_type_for_gda_type(column->get_g_type());
-          m_db_values[col] = Glom::Conversions::get_empty_value(glom_type);
+            //We don't just create a Gda::Value of the column's gda type,
+            //because we should use a NULL-type Gda::Value as the initial value for some fields:
+            const Field::glom_field_type glom_type = Field::get_glom_type_for_gda_type(column->get_g_type());
+            m_db_values[col] = Glom::Conversions::get_empty_value(glom_type);
+          }
         }
       }
     }
@@ -337,7 +350,7 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
     }
     else
     {
-      // Use a DataAccessWrapper to allow random access. This is necessary
+      // If using the sqlite backed, then use a DataAccessWrapper to allow random access. This is necessary
       // since we use move_to_row() on a created iterator in
       // fill_values_if_necessary(), which does not work if the iterator
       // does not support it (for example the one for Sqlite recordsets does
@@ -346,13 +359,15 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
       // a) make this code dependent on the database backend used.
       // b) fetch rows we perhaps don't need, if only the first few rows of
       // a table are accessed.
-      // TODO_Performance: The unnecessary (for PostgreSQL) extra indirection might theoretically make this slower.
-      m_gda_datamodel = Gnome::Gda::DataAccessWrapper::create(m_gda_datamodel);
+      // See See libgda bug https://bugzilla.gnome.org/show_bug.cgi?id=660344
+      ConnectionPool* connection = ConnectionPool::get_instance();
+      if(connection && !connection->get_backend_supports_cursor())
+        m_gda_datamodel = Gnome::Gda::DataAccessWrapper::create(m_gda_datamodel);
 
       //This doesn't work with cursor-based models: const int count = m_gda_datamodel->get_n_rows();
       //because rows count is -1 until we have iterated to the last row.
       Glib::RefPtr<Gnome::Gda::SqlBuilder> sql_query_without_sort = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_extra_join, type_sort_clause());
-      const int count = Base_DB::count_rows_returned_by(sql_query_without_sort);
+      const int count = DbUtils::count_rows_returned_by(sql_query_without_sort);
       if(count < 0)
       {
         std::cerr << G_STRFUNC << ": count is < 0" << std::endl;
@@ -969,7 +984,7 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
       {
         if(datamodel->get_n_rows())
         {
-          Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
+          const Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
 	  // This will probably fail on Windows, where a long is only 32 bits wide.
           total = static_cast<gulong>(value.get_int64()); //I discovered that it's a int64 by trying it.
         }
